﻿'    License: Microsoft Public License (Ms-PL) 
Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Lucene.Net
Imports Lucene.Net.Store
Imports Microsoft.WindowsAzure.StorageClient
Imports System.IO
Imports System.Diagnostics
Imports System.IO.Compression
Imports System.Threading


    Public Class AzureIndexOutput
        Inherits IndexOutput
        Private _azureDirectory As AzureDirectory
        Private _blobContainer As CloudBlobContainer
        Private _name As String
        Private _indexOutput As IndexOutput
        Private _fileMutex As Mutex
        Private _blob As CloudBlob
        Public ReadOnly Property CacheDirectory() As Lucene.Net.Store.Directory
            Get
                Return _azureDirectory.CacheDirectory
            End Get
        End Property

        Public Sub New(ByVal azureDirectory As AzureDirectory, ByVal blob As CloudBlob)
            _fileMutex = New Mutex(False, _name)
            _fileMutex.WaitOne()
            Try
                _azureDirectory = azureDirectory
                _blobContainer = _azureDirectory.BlobContainer
                _blob = blob
                _name = blob.Uri.Segments(blob.Uri.Segments.Length - 1)

                ' create the local cache one we will operate against...
                _indexOutput = CacheDirectory.CreateOutput(_name)
            Finally
                _fileMutex.ReleaseMutex()
            End Try
        End Sub

        Public Overrides Sub Flush()
            _indexOutput.Flush()
        End Sub

        Public Overrides Sub Close()
            _fileMutex.WaitOne()
            Try
                Dim fileName As String = _name

                ' make sure it's all written out
                _indexOutput.Flush()

                Dim originalLength As Long = _indexOutput.Length()
                _indexOutput.Close()

                Dim blobStream As Stream
#If COMPRESSBLOBS Then

				' optionally put a compressor around the blob stream
				If _azureDirectory.ShouldCompressFile(_name) Then
					' unfortunately, deflate stream doesn't allow seek, and we need a seekable stream
					' to pass to the blob storage stuff, so we compress into a memory stream
					Dim compressedStream As New MemoryStream()

					Try
						Dim indexInput As IndexInput = CacheDirectory.OpenInput(fileName)
						Using compressor As New DeflateStream(compressedStream, CompressionMode.Compress, True)
							' compress to compressedOutputStream
							Dim bytes(indexInput.Length() - 1) As Byte
							indexInput.ReadBytes(bytes, 0, CInt(Fix(bytes.Length)))
							compressor.Write(bytes, 0, CInt(Fix(bytes.Length)))
						End Using
						indexInput.Close()

						' seek back to beginning of comrpessed stream
						compressedStream.Seek(0, SeekOrigin.Begin)

						Debug.WriteLine(String.Format("COMPRESSED {0} -> {1} {2}% to {3}", originalLength, compressedStream.Length, (CSng(compressedStream.Length) / CSng(originalLength)) * 100, _name))
					Catch
						' release the compressed stream resources if an error occurs
						compressedStream.Dispose()
						Throw
					End Try

					blobStream = compressedStream
				Else
#End If
            blobStream = New StreamInput(CacheDirectory.OpenInput(fileName))

#If COMPRESSBLOBS Then
            End If
#End If

                Try
                    ' push the blobStream up to the cloud
                    _blob.UploadFromStream(blobStream)

                    ' set the metadata with the original index file properties
                    _blob.Metadata("CachedLength") = originalLength.ToString()
                    _blob.Metadata("CachedLastModified") = CacheDirectory.FileModified(fileName).ToString()
                    _blob.SetMetadata()

                    Debug.WriteLine(String.Format("PUT {1} bytes to {0} in cloud", _name, blobStream.Length))
                Finally
                    blobStream.Dispose()
                End Try

#If FULLDEBUG Then
				Debug.WriteLine(String.Format("CLOSED WRITESTREAM {0}", _name))
#End If
                ' clean up
                _indexOutput = Nothing
                _blobContainer = Nothing
                _blob = Nothing
                GC.SuppressFinalize(Me)
            Finally
                _fileMutex.ReleaseMutex()
            End Try
        End Sub

        Public Overrides Function Length() As Long
            Return _indexOutput.Length()
        End Function

        Public Overrides Sub WriteByte(ByVal b As Byte)
            _indexOutput.WriteByte(b)
        End Sub

        Public Overrides Sub WriteBytes(ByVal b() As Byte, ByVal length As Integer)
            _indexOutput.WriteBytes(b, length)
        End Sub

        Public Overrides Sub WriteBytes(ByVal b() As Byte, ByVal offset As Integer, ByVal length As Integer)
            _indexOutput.WriteBytes(b, offset, length)
        End Sub

        Public Overrides Function GetFilePointer() As Long
            Return _indexOutput.GetFilePointer()
        End Function

        Public Overrides Sub Seek(ByVal pos As Long)
            _indexOutput.Seek(pos)
        End Sub
    End Class