﻿'    License: Microsoft Public License (Ms-PL) 
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Lucene.Net
Imports Lucene.Net.Store
Imports Microsoft.WindowsAzure.StorageClient
Imports System.Diagnostics

    Public Class AzureLock
        Inherits Lock
        Private _lockFile As String
    Private _azureDirectory As AzureDirectory

    Public Sub New(ByVal lockFile As String, ByVal directory As AzureDirectory)
        _lockFile = lockFile
        _azureDirectory = directory
    End Sub

#Region "Lock methods"
        Public Overrides Function IsLocked() As Boolean
        Dim blob = _azureDirectory.BlobContainer.GetBlobReference(_lockFile)
            Try
                blob.FetchAttributes()
                Return True
            Catch err As StorageClientException
                If err.ErrorCode <> StorageErrorCode.ResourceNotFound Then
                    Trace.TraceError(err.ToString())
                End If
                Return False
            End Try
        End Function

        Public Overrides Function Obtain() As Boolean
            Try
                If IsLocked() Then
                    Return False
                End If

                Dim blob = _azureDirectory.BlobContainer.GetBlobReference(_lockFile)
                blob.UploadText("lock")
                Debug.WriteLine(String.Format("Obtained lock {0}", _lockFile))
                Return True
            Catch err As StorageClientException
                Select Case err.ErrorCode
                    Case StorageErrorCode.ContainerNotFound
                        ' container is missing, we should create it.
                        _azureDirectory.BlobContainer.Delete()
                        _azureDirectory.CreateContainer()
                        Return Me.Obtain()
                End Select
                Return False
            End Try
        End Function

        Public Overrides Sub Release()
            Debug.WriteLine(String.Format("Releasing lock {0}", _lockFile))
            Dim blob = _azureDirectory.BlobContainer.GetBlobReference(_lockFile)
            blob.DeleteIfExists()
        End Sub
#End Region

        Public Overrides Function ToString() As System.String
            Return "Lock@" & _lockFile
        End Function
    End Class