﻿'    License: Microsoft Public License (Ms-PL) 
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.IO
Imports Lucene.Net.Store

    ''' <summary>
    ''' Stream wrapper around an IndexOutput
    ''' </summary>
    Public Class StreamOutput
    Inherits Stream

    Private _Output As IndexOutput
    Public Property Output() As IndexOutput
        Get
            Return _Output
        End Get
        Set(ByVal value As IndexOutput)
            _Output = value
        End Set
    End Property

    Public Sub New(ByVal output As IndexOutput)
        Me.Output = output
    End Sub

    Public Overrides ReadOnly Property CanRead() As Boolean
        Get
            Return False
        End Get
    End Property

    Public Overrides ReadOnly Property CanSeek() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides ReadOnly Property CanWrite() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides Sub Flush()
        Output.Flush()
    End Sub

    Public Overrides ReadOnly Property Length() As Long
        Get
            Return Output.Length()
        End Get
    End Property

    Public Overrides Property Position() As Long
        Get
            Return Output.GetFilePointer()
        End Get
        Set(ByVal value As Long)
            Output.Seek(value)
        End Set
    End Property

    Public Overrides Function Read(ByVal buffer() As Byte, ByVal offset As Integer, ByVal count As Integer) As Integer
        Throw New NotImplementedException()
    End Function

    Public Overrides Function Seek(ByVal offset As Long, ByVal origin As SeekOrigin) As Long
        Select Case origin
            Case SeekOrigin.Begin
                Output.Seek(offset)
            Case SeekOrigin.Current
                Output.Seek(Output.GetFilePointer() + offset)
            Case SeekOrigin.End
                Throw New System.NotImplementedException()
        End Select
        Return Output.GetFilePointer()
    End Function

    Public Overrides Sub SetLength(ByVal value As Long)
        Throw New NotImplementedException()
    End Sub

    Public Overrides Sub Write(ByVal buffer() As Byte, ByVal offset As Integer, ByVal count As Integer)
        Output.WriteBytes(buffer, offset, count)
    End Sub

    Public Overrides Sub Close()
        Output.Flush()
        Output.Close()
        MyBase.Close()
    End Sub
End Class