﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient
Imports Lucene.Net
Imports Lucene.Net.Store
Imports Lucene.Net.Index
Imports Lucene.Net.Documents
Imports Lucene.Net.Util
Imports Lucene.Net.Analysis
Imports Lucene.Net.Analysis.Standard
Imports Lucene.Net.Search
Imports Lucene.Net.QueryParsers
Imports System.Diagnostics
Imports System.ComponentModel
Imports Lucene.Net.Store.Azure
Imports Microsoft.WindowsAzure.ServiceRuntime

Friend Class Program
    Public Shared fExit As Boolean = False

    Shared Sub MultiLamF(ByVal configName As String, ByVal configSetter As Func(Of String, Boolean))
        Try
            configSetter(RoleEnvironment.GetConfigurationSettingValue(configName))
        Catch e1 As Exception
            ' for a console app, reading from App.config
            configSetter(System.Configuration.ConfigurationManager.AppSettings(configName))
        End Try
    End Sub

    Shared Sub Main(ByVal args() As String)

        Dim MultiLamD As System.Action(Of String, Func(Of String, Boolean)) = AddressOf MultiLamF

        ' get settings from azure settings or app.config
        CloudStorageAccount.SetConfigurationSettingPublisher(MultiLamD)

        ' default AzureDirectory stores cache in local temp folder
        Dim azureDirectory As New AzureDirectory(CloudStorageAccount.FromConfigurationSetting("blobStorage"), "TestCatalog")
        Dim findexExists As Boolean = False
        Try
            findexExists = IndexReader.IndexExists(azureDirectory)
            If (findexExists) AndAlso IndexReader.IsLocked(azureDirectory) Then
                azureDirectory.ClearLock("write.lock")
            End If
        Catch e As Exception
            Console.WriteLine(e.ToString())
            Return
        End Try
        Dim indexWriter As New IndexWriter(azureDirectory, New Lucene.Net.Analysis.Standard.StandardAnalyzer(), (Not findexExists))
        indexWriter.SetRAMBufferSizeMB(10.0)
        indexWriter.SetUseCompoundFile(False)
        indexWriter.SetMaxMergeDocs(10000)
        indexWriter.SetMergeFactor(100)
        fExit = True
        For iDoc = 0 To 99
            If fExit Then
                Exit For
            End If
            If iDoc Mod 10 = 0 Then
                Console.WriteLine(iDoc)
            End If
            Dim doc As New Document()
            doc.Add(New Field("id", Date.Now.ToFileTimeUtc().ToString(), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO))
            doc.Add(New Field("Title", GeneratePhrase(10), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO))
            doc.Add(New Field("Body", GeneratePhrase(40), Field.Store.YES, Field.Index.TOKENIZED, Field.TermVector.NO))
            indexWriter.AddDocument(doc)
        Next iDoc
        Console.WriteLine("Total docs is {0}", indexWriter.DocCount())
        indexWriter.Close()

        Dim searcher As IndexSearcher
        Using TempAutoStopWatch As AutoStopWatch = New AutoStopWatch("Creating searcher")
            searcher = New IndexSearcher(azureDirectory) ' IndexReader.Open(
        End Using
        SearchForPhrase(searcher, "dog")
        SearchForPhrase(searcher, _random.Next(32768).ToString())
        SearchForPhrase(searcher, _random.Next(32768).ToString())
    End Sub


    Private Shared Sub SearchForPhrase(ByVal searcher As IndexSearcher, ByVal phrase As String)
        Using TempAutoStopWatch As AutoStopWatch = New AutoStopWatch(String.Format("Search for {0}", phrase))
            Dim parser As New Lucene.Net.QueryParsers.QueryParser("Body", New StandardAnalyzer())
            Dim query As Lucene.Net.Search.Query = parser.Parse(phrase)

            Dim hits As Hits = searcher.Search(query)
            Console.WriteLine("Found {0} results for {1}", hits.Length(), phrase)
            Dim max As Integer = hits.Length()
            If max > 100 Then
                max = 100
            End If
            For i = 0 To max - 1
                Console.WriteLine(hits.Doc(i).GetField("Title").StringValue())
            Next i
        End Using
    End Sub

    Private Shared _random As New Random(CInt(Fix(Date.Now.Ticks)))
    Private Shared sampleTerms() As String = {"dog", "cat", "car", "horse", "door", "tree", "chair", "microsoft", "apple", "adobe", "google", "golf", "linux", "windows", "firefox", "mouse", "hornet", "monkey", "giraffe", "computer", "monitor", "steve", "fred", "lili", "albert", "tom", "shane", "gerald", "chris", "love", "hate", "scared", "fast", "slow", "new", "old"}

    Private Shared Function GeneratePhrase(ByVal MaxTerms As Integer) As String
        Dim phrase As New StringBuilder()
        Dim nWords As Integer = 2 + _random.Next(MaxTerms)
        For i = 0 To nWords - 1
            phrase.AppendFormat(" {0} {1}", sampleTerms(_random.Next(sampleTerms.Length)), _random.Next(32768).ToString())
        Next i
        Return phrase.ToString()
    End Function

End Class

Public Class AutoStopWatch
    Implements IDisposable
    Private _stopwatch As Stopwatch
    Private _message As String
    Public Sub New(ByVal message As String)
        _message = message
        Debug.WriteLine(String.Format("{0} starting ", message))
        _stopwatch = Stopwatch.StartNew()
    End Sub

#Region "IDisposable Members"
    Public Sub Dispose() Implements IDisposable.Dispose

        _stopwatch.Stop()
        Dim ms As Long = _stopwatch.ElapsedMilliseconds

        Debug.WriteLine(String.Format("{0} Finished {1} ms", _message, ms))
    End Sub
#End Region
End Class
