﻿Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Linq
Imports System.Net
Imports System.Threading
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.Diagnostics
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Microsoft.WindowsAzure.StorageClient

Imports Lucene.Net.Index
Imports Lucene.Net.Store
' namespace collision with DocumentFormat.OpenXml so I gave it an alias
Imports LuceneDocs = Lucene.Net.Documents
Imports Lucene.Net.Analysis.Standard
Imports Lucene.Net.Store.Azure

Imports System.IO
Imports DocumentFormat.OpenXml.Packaging
Imports DocumentFormat.OpenXml.Wordprocessing
Imports Lucene.Net.Search
Imports Lucene.Net.QueryParsers

Public Class WorkerRole
    Inherits RoleEntryPoint

    Public Overrides Sub Run()

        ' This is a sample implementation for WorkerRole1. Replace with your logic.
        Trace.WriteLine("WorkerRole1 entry point called.", "Information")
        CloudStorageAccount.SetConfigurationSettingPublisher(Sub(configName, configSettingPublisher)
                                                                 Dim configValue As String = RoleEnvironment.GetConfigurationSettingValue(configName)
                                                                 configSettingPublisher(configValue)
                                                             End Sub)

        Index(True)

        While (True)
            Thread.Sleep(10000)
            Trace.WriteLine("Working", "Information")
        End While

    End Sub

    Public Overrides Function OnStart() As Boolean

        ' Set the maximum number of concurrent connections 
        ServicePointManager.DefaultConnectionLimit = 12

        ' For information on handling configuration changes
        ' see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

        Return MyBase.OnStart()

    End Function

    Public Sub Index(ByVal DoFull As Boolean)
        Dim storageAccount As CloudStorageAccount = CloudStorageAccount.FromConfigurationSetting("StorageAccount")
        Dim BlobClient As CloudBlobClient = storageAccount.CreateCloudBlobClient()

        ' open AzureDirectory which contains the index
        Dim azureDirectory As New AzureDirectory(storageAccount, "CloudIndex")

        ' need to check segments.gen last modified date and filter documents to index by that timestamp
        Dim LastModified As New Date(IndexReader.LastModified(azureDirectory), DateTimeKind.Utc)
        Dim GenerateIndex As Boolean = Not IndexReader.IndexExists(azureDirectory)
        DoFull = GenerateIndex

        ' loop and fetch the information for each one
        ' this needs to be managed for memory pressure, but for sample I'll do all in one pass
        Dim indexWriter As New IndexWriter(azureDirectory, New StandardAnalyzer(Lucene.Net.Util.Version.LUCENE_29), GenerateIndex, IndexWriter.MaxFieldLength.UNLIMITED)

        Dim options As New BlobRequestOptions()
        options.BlobListingDetails = BlobListingDetails.Metadata
        options.UseFlatBlobListing = True

        ' get container to be indexed
        Dim Container As CloudBlobContainer = BlobClient.GetContainerReference("documents")
        Container.CreateIfNotExist()

        For Each currentBlob In Container.ListBlobs(options)
            Dim blobRef As CloudBlob = Container.GetBlobReference(currentBlob.Uri.ToString())
            blobRef.FetchAttributes(options)
            ' add doc to index if it is newer than index or doing a full index
            If LastModified < blobRef.Properties.LastModifiedUtc OrElse DoFull Then
                Dim curBlob As DocumentToIndex = GetDocumentData(currentBlob.Uri.ToString())

                ' docs.Add(curBlob)

                AddToCatalog(indexWriter, curBlob)
            End If
        Next currentBlob

        If DoFull Then
            indexWriter.Optimize()
        End If

        indexWriter.Commit()
        indexWriter.Close()

    End Sub

    Public Function GetDocumentData(ByVal docUri As String) As DocumentToIndex
        Dim doc As New DocumentToIndex()

        'Dim stream As Stream = File.Open(docUri, FileMode.Open)
        Dim response = WebRequest.Create(docUri).GetResponse()
        Dim stream As Stream = response.GetResponseStream()

        'can't open directly from URI, because it won't support seeking
        'so move it to a "local" memory stream
        Dim localStream As Stream = New MemoryStream()
        stream.CopyTo(localStream)

        'parse doc name
        doc.Name = docUri.Substring(docUri.LastIndexOf("/") + 1)
        doc.Uri = docUri

        Select Case doc.Name.Substring(doc.Name.LastIndexOf(".") + 1)
            Case "docx"
                Dim wordprocessingDocument As WordprocessingDocument = WordprocessingDocument.Open(localStream, False)
                doc.Body = wordprocessingDocument.MainDocumentPart.Document.Body.InnerText
                wordprocessingDocument.Close()
                'TODO:  Still incomplete
            Case "pptx"
                'probably want to create a generic fo DocToIndex and use it to create a pptx specific that allows slide 
                'specific indexing
                Dim pptDoc As PresentationDocument = PresentationDocument.Open(localStream, False)
                For Each slide In pptDoc.PresentationPart.SlideParts
                    'iterate through slides
                Next slide
            Case Else

        End Select

        stream.Close()
        Return doc
    End Function

    Public Sub RemoveFromCatalog(ByVal docs As List(Of DocumentToIndex), ByVal azureDir As AzureDirectory)

        Dim indexReader As IndexReader = indexReader.Open(azureDir)
        For Each currentDocument In docs

            Dim deleteTerm As New Term("Uri", currentDocument.Uri)
            Dim deletedCount As Long = indexReader.DeleteDocuments(deleteTerm)

        Next currentDocument

        indexReader.Commit()
        indexReader.Close()
    End Sub

    Public Sub AddToListCatalog(ByVal indexWriter As IndexWriter, ByVal docs As List(Of DocumentToIndex))
        For Each currentDocument In docs
            AddToCatalog(indexWriter, currentDocument)
        Next currentDocument

    End Sub

    Public Sub AddToCatalog(ByVal indexWriter As IndexWriter, ByVal currentDocument As DocumentToIndex)

        Dim deleteTerm As New Term("Uri", currentDocument.Uri)

        Dim doc As New LuceneDocs.Document()
        doc.Add(New LuceneDocs.Field("Uri", currentDocument.Uri, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.NOT_ANALYZED, LuceneDocs.Field.TermVector.NO))
        doc.Add(New LuceneDocs.Field("Title", currentDocument.Name, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.ANALYZED, LuceneDocs.Field.TermVector.NO))
        doc.Add(New LuceneDocs.Field("Body", currentDocument.Body, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.ANALYZED, LuceneDocs.Field.TermVector.NO))

        indexWriter.UpdateDocument(deleteTerm, doc)
    End Sub
End Class

Public Class DocumentToIndex
    Public Body As String
    Public Name As String
    Public Uri As String
    Public Id As String
End Class
